package com.tzld.longarticle.recommend.server.remote;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.HttpPoolFactory;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recall.RecallParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * @author dyp
 */
@Service
@Slf4j
public class AIGCRemoteService {
    private final CloseableHttpClient client = HttpPoolFactory.aigcPool();
    private static final String aigcGetAllContentUrl = "http://aigc-api.cybertogether.net/aigc/publish/content/gzhWaitingPublishContent";


    public List<Content> getAllContent(RecallParam param) {
        long start = System.currentTimeMillis();
        List<Content> result = new ArrayList<>();
        JSONObject bodyParam = new JSONObject();
        JSONObject bodyParamParams = new JSONObject();
        bodyParamParams.put("accountId", param.getAccountId());
        bodyParamParams.put("planId", param.getPlanId());
        bodyParam.put("params", bodyParamParams);
        try {
            HttpPost httpPost = new HttpPost(aigcGetAllContentUrl);
            StringEntity stringEntity = new StringEntity(bodyParam.toJSONString(), StandardCharsets.UTF_8);
            httpPost.setHeader("Content-Type", "application/json;charset=UTF-8");
            httpPost.setEntity(stringEntity);
            CloseableHttpResponse response = client.execute(httpPost);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    // log.info("getAllContent AIGC返回的数据：{}", responseBody);
                    JSONObject jsonObject = JSONObject.parseObject(responseBody);
                    if (jsonObject.getInteger("code") == 0) {
                        JSONArray data = jsonObject.getJSONArray("data");
                        result.addAll(JSONArray.parseArray(data.toJSONString(), Content.class));
                    }
                }
            }
        } catch (Exception e) {
            log.error("getAllContent error", e);
        }
        log.info("getAllContent耗时：{}", System.currentTimeMillis() - start);
        return result;
    }

    public Set<String> historyTitle() {
        return Collections.emptySet();
    }

}
