package com.tzld.longarticle.recommend.server.service.filter.strategy;

import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.dto.ContentHisPublishArticle;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.filter.FilterStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;

@Component
@Slf4j
public class LowScoreStrategy implements FilterStrategy {

    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    AccountContentPoolConfigService accountContentPoolConfigService;

    @Override
    public FilterResult filter(FilterParam param) {
        long start = System.currentTimeMillis();
        FilterResult filterResult = new FilterResult();
        filterResult.setContentIds(new ArrayList<>());
        filterResult.setFilterContent(new ArrayList<>());
        for (Content content : param.getContents()) {
            String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
            if (!contentPools[2].equals(content.getContentPoolType())) {
                filterResult.getContentIds().add(content.getId());
                continue;
            }
            if (CollectionUtils.isEmpty(content.getHisPublishArticleList())) {
                filterResult.getContentIds().add(content.getId());
                continue;
            }
            long publishCount = content.getHisPublishArticleList().stream().filter(ContentHisPublishArticle::isInnerAccount).count();
            if (publishCount == 0) {
                continue;
            }
            int hisViewCount = content.getHisPublishArticleList().stream().filter(ContentHisPublishArticle::isInnerAccount)
                    .mapToInt(ContentHisPublishArticle::getViewCount).sum();
            int hisAvgViewCount = content.getHisPublishArticleList().stream().filter(ContentHisPublishArticle::isInnerAccount)
                    .mapToInt(ContentHisPublishArticle::getAvgViewCount).sum();
            double rate = (hisViewCount * 1.0) / hisAvgViewCount;
            int sumAvgViewCount = content.getHisPublishArticleList().stream().filter(ContentHisPublishArticle::isInnerAccount)
                    .mapToInt(ContentHisPublishArticle::getAvgViewCount).sum();
            if (((publishCount >= 2 && rate < 0.8)
                    || (publishCount == 1 && rate < 0.5)) && sumAvgViewCount > 100) {
                content.setFilterReason("低评分内容");
                filterResult.getFilterContent().add(content);
            } else {
                filterResult.getContentIds().add(content.getId());
            }
        }
        return filterResult;
    }
}
