package com.tzld.longarticle.recommend.server.service.rank;


import com.tzld.longarticle.recommend.server.common.enums.ContentPoolEnum;
import com.tzld.longarticle.recommend.server.common.enums.FeishuRobotIdEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.strategy.SimilarityStrategy;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import com.tzld.longarticle.recommend.server.util.feishu.FeishuMessageSender;
import org.apache.commons.collections4.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
public interface RankStrategy {

    RankResult rank(RankParam param);

    public static void deduplication(List<Content> result, Map<String, List<Content>> contentMap, String[] publishPool) {
        List<String> titles = new ArrayList<>();
        for (int i = 0; i < result.size(); i++) {
            if (TitleSimilarCheckUtil.isDuplicateContent(result.get(i).getTitle(), titles)) {
                if (i == 0) {
                    for (Content content : contentMap.get(publishPool[0])) {
                        if (!TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), titles)) {
                            result.set(i, content);
                            titles.add(content.getTitle());
                            break;
                        }
                    }
                } else if (i == 1) {
                    for (Content content : contentMap.get(publishPool[1])) {
                        if (!TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), titles)) {
                            result.set(i, content);
                            titles.add(content.getTitle());
                            break;
                        }
                    }
                } else {
                    for (Content content : contentMap.get(publishPool[2])) {
                        if (!TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), titles)) {
                            result.set(i, content);
                            titles.add(content.getTitle());
                            break;
                        }
                    }
                }
            } else {
                titles.add(result.get(i).getTitle());
            }
        }
    }

    public static ScoreParam convertToScoreParam(RankParam param) {
        ScoreParam scoreParam = new ScoreParam();
        scoreParam.setGhId(param.getGhId());
        scoreParam.setAccountName(param.getAccountName());
        scoreParam.setContents(param.getContents());
        scoreParam.setStrategy(param.getStrategy());
        scoreParam.setScene(param.getScene());
        return scoreParam;
    }


    public static List<Content> deduplication(List<Content> contents) {
        List<String> titles = new ArrayList<>();
        List<Content> result = new ArrayList<>();
        // 遍历所有列表
        for (String contentPool : ContentPoolEnum.getOrderContentPool()) {
            for (Content c : contents) {
                if (!contentPool.equals(c.getContentPoolType())) {
                    continue;
                }
                if (!TitleSimilarCheckUtil.isDuplicateContent(c.getTitle(), titles)) {
                    result.add(c);
                    titles.add(c.getTitle());
                }
            }
        }

        return result;
    }

    static boolean SimilarityScoreZero(List<RankItem> items, RankParam param, List<Article> articleList) {
        if (CollectionUtils.isNotEmpty(items)
                && items.get(0).getScoreMap().containsKey(SimilarityStrategy.class.getSimpleName())
                && items.get(0).getScoreMap().get(SimilarityStrategy.class.getSimpleName()) == 0) {
                FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.recommend.getRobotId(),
                        "内容评分为0\n"
                                + "ghId: " + param.getGhId() + "\n"
                                + "账号名称: " + param.getAccountName() + "\n"
                                + "策略: " + param.getStrategy());
                return true;
        }
        return false;
    }

    static void sendFeishuFirstPoolEmpty(RankParam param, String contentPool) {
        FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.recommend.getRobotId(),
                "内容池1为空\n"
                        + "ghId: " + param.getGhId() + "\n"
                        + "账号名称: " + param.getAccountName() + "\n"
                        + "内容池: " + contentPool + "\n"
                        + "策略: " + param.getStrategy());
    }

}
