package com.tzld.longarticle.recommend.server.service.score.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.reflect.TypeToken;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticlePreDistributeAccountRepository;
import com.tzld.longarticle.recommend.server.model.entity.crawler.ArticlePreDistributeAccount;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.score.Score;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * @author dyp
 */
@Component
@Slf4j
public class AccountPreDistributeStrategy implements ScoreStrategy {

    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;
    @Autowired
    private ArticlePreDistributeAccountRepository articlePreDistributeAccountRepository;

    @ApolloJsonValue("${accountPreDistributeWeightConfig:{}}")
    private Map<String, Integer> accountPreDistributeWeightMap;

    @Override
    public List<Score> score(ScoreParam param) {
        long start = System.currentTimeMillis();
        List<Score> scores = new ArrayList<>();
        List<Content> contents = param.getContents();
        Set<String> articles = new HashSet<>();
        String[] pools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        if (pools.length >= 3) {
            try {
                ArticlePreDistributeAccount optional = articlePreDistributeAccountRepository.findFirstByGhIdOrderByDateDesc(param.getGhId());
                if (optional != null && StringUtils.hasText(optional.getArticleList())) {
                    List<String[]> list = JSONUtils.fromJson(optional.getArticleList(), new TypeToken<List<String[]>>() {
                    }, Collections.emptyList());
                    for (String[] s : list) {
                        articles.add(s[0]);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        Integer weight = getPreDistributeWeight(param.getAccountName());
        for (Content content : contents) {
            Score score = new Score();
            score.setStrategy(this);
            if (pools[2].equals(content.getContentPoolType())) {
                if (articles.contains(content.getCrawlerChannelContentId())) {
                    score.setContentId(content.getId());
                    score.setScore(Double.valueOf(weight));
                }
            }
            if (StringUtils.hasText(score.getContentId())) {
                scores.add(score);
            }
        }
        log.info("AccountPreDistributeStrategy cost:{}", System.currentTimeMillis() - start);
        return scores;
    }

    public Integer getPreDistributeWeight(String accountName) {
        if (accountPreDistributeWeightMap.containsKey(accountName)) {
            return accountPreDistributeWeightMap.get(accountName);
        }
        return accountPreDistributeWeightMap.get("default");
    }

}
