package com.tzld.longarticle.recommend.server.service.score.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.service.score.Score;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.NormalizationUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Component
@Slf4j
public class ContentPoolStrategy implements ScoreStrategy {

    @ApolloJsonValue("${contentPoolScore:{}}")
    private Map<String, Double> contentPoolScore;

    @Override
    public List<Score> score(ScoreParam param) {
        long start = System.currentTimeMillis();

        if (CollectionUtils.isEmpty(param.getContents())) {
            return Collections.emptyList();
        }

        double min = contentPoolScore.values().stream()
                .min(Double::compareTo)
                .orElse(0.0);
        double max = contentPoolScore.values().stream()
                .max(Double::compareTo)
                .orElse(0.0);

        List<Score> scores = CommonCollectionUtils.toList(param.getContents(), c -> {
            Score score = new Score();
            score.setContentId(c.getId());
            double val = contentPoolScore.getOrDefault(c.getContentPoolType(), 0.0);
            score.setScore(NormalizationUtils.minMax(val, min, max));
            score.setStrategy(this);
            return score;
        });
        return scores;
    }

}
